@@ -4,9 +4,18 @@ g3a_grow_vec_rotate <- g3_native(r = function (vec, a) {
         function (i) vec[i:(i+length(vec) - 1)],
         numeric(length(vec)))
     out[is.na(out)] <- vec[length(vec)]  # Overflowed entries should be capped at the final one
-    out
-}, cpp = '[](vector<Type> vec, int a) -> array<Type> {
-    array<Type> out(vec.size(), a);
+	    out
+}, cpp = '
+template<typename T, TYPE_IS_SCALAR(T)>
+array<T> __fn__(T vec, int a) {
+    array<T> out(1, a);
+    // NB: Treat scalar as a 1x vector (for dynlen)
+    out.setConstant(vec);
+    return out;
+}
+template<typename T>
+array<typename T::value_type> __fn__(const Eigen::DenseBase<T>& vec, int a) {
+    array<typename T::value_type> out(vec.size(), a);
     for (int i = 0 ; i < vec.size(); i++) {
         for (int j = 0 ; j < a; j++) {
             out(i, j) = vec(j + i < vec.size() ? j + i : vec.size() - 1);
@@ -16,8 +25,17 @@ g3a_grow_vec_rotate <- g3_native(r = function (vec, a) {
 }')
 g3a_grow_vec_extrude <- g3_native(r = function (vec, a) {
     array(vec, dim = c(length(vec), a))
-}, cpp = '[](vector<Type> vec, int a) -> array<Type> {
-    array<Type> out(vec.size(), a);
+}, cpp = '
+template<typename T, TYPE_IS_SCALAR(T)>
+array<T> __fn__(T vec, int a) {
+    array<T> out(1, a);
+    // NB: Treat scalar as a 1x vector (for dynlen)
+    out.setConstant(vec);
+    return out;
+}
+template<typename T>
+array<typename T::value_type> __fn__(const Eigen::DenseBase<T>& vec, int a) {
+    array<typename T::value_type> out(vec.size(), a);
     out = vec.replicate(a, 1);
     return out;
 }')
@@ -47,8 +65,8 @@ g3a_grow_weightsimple <- function (
     # growmemberfunctions.cc:61 - Make a l --> l' matrix of weight increases,
     # NB: Have to multiply by alpha_f last, otherwise TMB thinks the result should be scalar
     f_substitute(~(
-        g3a_grow_vec_rotate(stock__midlen^beta_f, maxlengthgroupgrowth + 1) -
-        g3a_grow_vec_extrude(stock__midlen^beta_f, maxlengthgroupgrowth + 1)
+        g3a_grow_vec_rotate(stock__midlen^beta_f, maxlengthgroupgrowth + 1L) -
+        g3a_grow_vec_extrude(stock__midlen^beta_f, maxlengthgroupgrowth + 1L)
     ) * (alpha_f), list(alpha_f = alpha_f, beta_f = beta_f))
 }
 
@@ -77,7 +95,7 @@ g3a_grow_weight_multspec <- function(
         p4 *
         (stock_ss(stock__wgt))^p5 *
         (stock_ss(stock__feedinglevel) - p6) *
-        (p7 * temperature + p8), maxlengthgroupgrowth + 1)
+        (p7 * temperature + p8), maxlengthgroupgrowth + 1L)
 }
 
 g3a_grow_length_weightjones <- function(
@@ -119,8 +137,8 @@ g3a_grow_weight_weightjones <- function(
         end = NULL )
     # NB: Pretty easy to have negative growth with nonsense parameters, thus avoid_zero()
     ~g3a_grow_vec_extrude(avoid_zero(cur_step_size * (
-        ((max_consumption * stock_ss(stock__feedinglevel)) / (q0 * (stock_ss(stock__wgt))^q1)) -
-        q2 * (stock_ss(stock__wgt))^q3 * exp(q4 * temperature + q5) )), maxlengthgroupgrowth + 1)
+        ((max_consumption * stock_ss(stock__feedinglevel)) / (q0 * avoid_zero(stock_ss(stock__wgt))^q1)) -
+        q2 * (stock_ss(stock__wgt))^q3 * exp(q4 * temperature + q5) )), maxlengthgroupgrowth + 1L)
 }
 
 # Returns bbinom growth implementation formulae
