@@ -13,4 +13,5 @@ g3_action_order <- list(
   straying = 9L,
   likelihood = 10L,
   report = 11L,
-  age = 12L )
+  age = 12L,
+  quota = 13L )
