@@ -19,9 +19,8 @@ g3l_random_dnorm <- function (
         log_f = TRUE,
         period = 'auto',
         nll_breakdown = FALSE,
-        weight = substitute(
-            g3_param(n, optimise = FALSE, value = 1),
-            list(n = paste0(nll_name, "_weight"))),
+        weight = g3_parameterized(paste0(nll_name, "_weight"),
+            optimise = FALSE, value = 1),
         run_at = g3_action_order$likelihood) {
     stopifnot(period %in% c('year', 'step', 'single', 'auto'))
     stopifnot(is.logical(log_f))
@@ -67,9 +66,8 @@ g3l_random_walk <- function (
         log_f = TRUE,
         period = 'auto',
         nll_breakdown = FALSE,
-        weight = substitute(
-            g3_param(n, optimise = FALSE, value = 1),
-            list(n = paste0(nll_name, "_weight"))),
+        weight = g3_parameterized(paste0(nll_name, "_weight"),
+            optimise = FALSE, value = 1),
         run_at = g3_action_order$likelihood) {
     stopifnot(period %in% c('year', 'step', 'single', 'auto'))
     stopifnot(is.logical(log_f))
