@@ -63,6 +63,17 @@ cpp_code <- function(in_call, in_envir, indent = "\n    ", statement = FALSE, ex
             return(TRUE)
         }
 
+        # Common functions returning scalar from any arrays
+        if (is.call(c_val) && as.character(c_val[[1]]) %in% c("sum", "prod", "mean", "g3_idx")) {
+            return(TRUE)
+        }
+
+        # Operators that will return a same-length array
+        if (is.call(c_val) && as.character(c_val[[1]]) %in% c("-", "+", "*", "/", "%/%", "%%")) return(all(vapply(
+            tail(c_val, -1),
+            value_is_scalar,
+            logical(1) )))
+
         # Dunno.
         return(fallback)
     }
@@ -181,7 +192,7 @@ cpp_code <- function(in_call, in_envir, indent = "\n    ", statement = FALSE, ex
 
         # Are we assigning to an array-like object?
         if (is.call(assign_lhs) && assign_lhs[[1]] == '[') {
-            if (grepl('.transpose()', cpp_code(assign_lhs, in_envir, next_indent))) {
+            if (grepl('.transpose()', cpp_code(assign_lhs, in_envir, next_indent), fixed = TRUE)) {
                 stop("Can't assign to this subset under TMB (.transpose() isn't by reference): ", deparse1(assign_lhs))
             }
             # i.e. there is at least one "missing" in the subset, i.e. we're not going to put a (0) on it
@@ -213,9 +224,9 @@ cpp_code <- function(in_call, in_envir, indent = "\n    ", statement = FALSE, ex
         # Add += operators if possible
         assign_op <- "="
         if (is.call(assign_rhs)
+                && length(assign_rhs) == 3  # NB: Do this first so x <- y() doesn't trip us up
                 && identical(assign_lhs, assign_rhs[[2]])
-                && as.character(assign_rhs[[1]]) %in% c("+", "-", "*", "/")
-                && length(assign_rhs) == 3) {
+                && as.character(assign_rhs[[1]]) %in% c("+", "-", "*", "/") ) {
             # Operating on iself, use a += operation
             assign_op <- paste0(assign_rhs[[1]], "=")
             assign_rhs <- assign_rhs[[3]]
@@ -607,10 +618,12 @@ cpp_code <- function(in_call, in_envir, indent = "\n    ", statement = FALSE, ex
 
     if (call_name == "sum") {
         # NB: TMB has a sum(), but it doesn't work in all cases and this is all it does anyway.
+        if (value_is_scalar(in_call[[2]])) return(cpp_code(in_call[[2]], in_envir, next_indent))
         return(paste0("(", cpp_code(in_call[[2]], in_envir, next_indent), ").sum()"))
     }
 
     if (call_name == "prod") {
+        if (value_is_scalar(in_call[[2]])) return(cpp_code(in_call[[2]], in_envir, next_indent))
         return(paste0("(", cpp_code(in_call[[2]], in_envir, next_indent), ").prod()"))
     }
 
@@ -746,16 +759,20 @@ g3_to_tmb <- function(actions, trace = FALSE, strict = FALSE) {
             df_template <- function (name, dims = c(1), sub_param_idx = NULL) {
                 # Extract named args from g3_param() call
                 value <- find_arg('value', 0, sub_param_idx = sub_param_idx)
-                optimise <- find_arg('optimise', !find_arg('random', FALSE))  # i.e. default is opposite of random
                 random <- find_arg('random', FALSE)
                 lower <- as.numeric(find_arg('lower', NA, sub_param_idx = sub_param_idx))
                 upper <- as.numeric(find_arg('upper', NA, sub_param_idx = sub_param_idx))
+                optimise <- find_arg('optimise', is.finite(lower) && is.finite(upper) && isFALSE(random))
                 parscale <- as.numeric(find_arg('parscale', NA, sub_param_idx = sub_param_idx))
+                type <- as.character(find_arg('type', character(0)))
                 source <- as.character(find_arg('source', as.character(NA)))
 
+                if (x[[1]] == "g3_param_array") type <- c(type, "ARRAY")
+                if (x[[1]] == "g3_param_vector") type <- c(type, "VECTOR")
+
                 data.frame(
                     switch = name,  # NB: This should be pre-C++ mangling
-                    type = if (x[[1]] == "g3_param_array") "ARRAY" else if (x[[1]] == "g3_param_vector") "VECTOR" else "",
+                    type = paste(type, collapse = ":"),
                     value = I(structure(
                         # NB: Has to be a list column because values might be vectors
                         list(if (identical(dims, c(1))) value else array(value, dim = dims)),
@@ -912,7 +929,7 @@ g3_to_tmb <- function(actions, trace = FALSE, strict = FALSE) {
                     # NB: bool -> int, as array<bool> doesn't REPORT() (tests/test-action_time.R, R/test_utils.R)
                     cpp_type <- 'int'
                 } else {
-                    stop("Don't know how to define ", var_name, " = ", paste(capture.output(str(var_val)), collapse = "\n    "))
+                    stop("Don't know how to define ", var_name, " = ", paste(utils::capture.output(utils::str(var_val)), collapse = "\n    "))
                 }
                 if (is.array(var_val)) {
                     cpp_type <- paste0('array<', cpp_type, '>')
@@ -1035,6 +1052,7 @@ Type objective_function<Type>::operator() () {
 
     # Make sure we include TMB
     out <- c(
+        sprintf("// Model generated with gadget3 %s", package_version_info("gadget3")),
         "#include <TMB.hpp>",
         "#include <numeric>",  # Required for std::partial_sum, std::adjacent_difference
         "", out)
@@ -1093,6 +1111,7 @@ g3_tmb_adfun <- function(
             framework = getOption("gadget3.tmb.framework", default = "TMBad") ),
         ...) {
     model_params <- attr(cpp_code, 'parameter_template')
+    if ("Type" %in% ...names()) warning("'Type' isn't a TMB::MakeADFun parameter, use 'type'")
 
     if (!dir.exists(work_dir)) dir.create(work_dir, showWarnings = FALSE, recursive = TRUE)
 
@@ -1124,10 +1143,8 @@ g3_tmb_adfun <- function(
 
     for (i in seq_len(nrow(parameters))) {
         val <- parameters[i, 'value'][[1]]
-        if (parameters[i, 'type'] == "ARRAY" && !is.array(val)) stop("Parameter ", parameters[i, 'switch'], " not an array")
-        if (parameters[i, 'type'] == "MATRIX" && !is.matrix(val)) stop("Parameter ", parameters[i, 'switch'], " not a matrix")
-        # What can we test if parameters[n, 'type'] == "VECTOR"?
-        if (parameters[i, 'type'] == "" && length(val) != 1) stop("Parameter ", parameters[i, 'switch'], " should be a single value")
+        if (grepl("(^|:)ARRAY(:|$)", parameters[i, 'type']) && !is.array(val)) stop("Parameter ", parameters[i, 'switch'], " not an array")
+        if (grepl("(^|:)MATRIX(:|$)", parameters[i, 'type']) && !is.matrix(val)) stop("Parameter ", parameters[i, 'switch'], " not a matrix")
     }
 
     if (!any(parameters$optimise) && !any(parameters$random)) {
@@ -1137,6 +1154,8 @@ g3_tmb_adfun <- function(
     tmb_parameters <- structure(
         parameters$value,
         names = cpp_escape_varname(parameters$switch))
+    logarithmic <- grepl("(^|:)LOG(:|$)", parameters$type)
+    tmb_parameters[logarithmic] <- lapply(tmb_parameters[logarithmic], log)
 
     # optimise=F & random=F parameters should be added to fixed map.
     tmb_map <- lapply(parameters[parameters$optimise == FALSE & parameters$random == FALSE, 'switch'], function (n) {
@@ -1151,6 +1170,7 @@ g3_tmb_adfun <- function(
         tmb_random <- NULL
     } else {
         tmb_random <- cpp_escape_varname(parameters[parameters$random == TRUE, 'switch'])
+        if (length(tmb_random) == 0) tmb_random <- NULL
     }
 
     if (any(parameters$random & parameters$optimise)) {
@@ -1284,17 +1304,64 @@ g3_tmb_adfun <- function(
         report_patch(fn$orig_simulate(...))
     }
 
-    # With Type = "Fun", fn$par sometimes isn't set. Bodge around it
+    # With type = "Fun", fn$par sometimes isn't set. Bodge around it
     if (is.null(fn$par)) fn$par <- g3_tmb_par(parameters)
     return(fn)
 }
 
+# Make a simple function only capable of double evaluation, for e.g. projections
+g3_tmb_fn <- function (
+        cpp_code,
+        def.parameters = attr(cpp_code, 'parameter_template'),
+        ... ) {
+    # Pretend we're optimising everything, so all parameters can be adjusted. There is no tape to worry about.
+    def.parameters$optimise <- TRUE
+    def.parameters$random <- FALSE
+
+    obj.fn <- g3_tmb_adfun(cpp_code, def.parameters, type = "Fun", ...)
+
+    # Wrapping function that hides TMB magic, works like the R function
+    out <- function (parameters = NULL) {
+        p <- def.parameters
+        if (!is.null(parameters)) {
+            # Splice in provided parameters, after converting back to list
+            if (is.data.frame(parameters)) parameters <- parameters$value
+            if (!is.list(parameters)) parameters <- g3_tmb_relist(def.parameters, parameters)
+            p$value[names(parameters)] <- parameters
+        }
+
+        # Run gen_dimnames & repopulate any dynamic dims, re-patching report_patch
+        # NB: We have to do this here, as the project_years parameter is reliant on this re-patching
+        #     (which ordinarily won't be optimise=TRUE, and you'd have to re-run g3_tmb_adfun)
+        dyndims <- attributes(attr(cpp_code, 'report_gen_dimnames')(p))
+        report_dimnames <- attr(cpp_code, 'report_dimnames')
+
+        for (dimname in names(dyndims)) {
+            for (var_name in names(report_dimnames)) {
+                if (dimname %in% names(report_dimnames[[var_name]]) && is.null(report_dimnames[[var_name]][[dimname]])) {
+                    report_dimnames[[var_name]][[dimname]] <- dyndims[[dimname]]
+                }
+            }
+        }
+        environment(environment(obj.fn$report)$report_patch)$report_dimnames <- report_dimnames
+
+        # NB: Using $simulate instead of $report will
+        # * Enable SIMULATE blocks (which we don't use)
+        # * Save RNG state back to R after a run (which is slightly more intuitive than successive runs producing the same result)
+        obj.fn$simulate(g3_tmb_par(p), complete = FALSE)
+    }
+    class(out) <- c("g3_tmb_fn", class(out))
+    attr(out, 'parameter_template') <- def.parameters
+    attr(out, 'model_data') <- attr(cpp_code, "model_data")
+    return(out)
+}
+
 # Turn parameter template into vectors of upper/lower bounds
 g3_tmb_bound <- function (parameters, bound, include_random = FALSE) {
     # Get all parameters we're thinking of optimising
     p <- parameters[
         (if (include_random) parameters$random else FALSE) |
-        parameters$optimise, c('switch', 'value', bound)]
+        parameters$optimise, c('switch', 'value', bound, 'type')]
 
     if (bound == 'value') {
         out <- p$value
@@ -1307,6 +1374,10 @@ g3_tmb_bound <- function (parameters, bound, include_random = FALSE) {
     }
     names(out) <- cpp_escape_varname(p$switch)
 
+    # Take the log of any logarithmic parameters
+    logarithmic <- grepl("(^|:)LOG(:|$)", p$type)
+    out[logarithmic] <- lapply(out[logarithmic], log)
+
     # Unlist the result to condense list back to vector
     unlist(out)
 }
@@ -1350,5 +1421,10 @@ g3_tmb_relist <- function (parameters, par) {
         parameters$optimise)], out)
     # Re-order to match template list
     out <- out[names(parameters$value)]
+
+    # Convert any logarithmic params back to linear space
+    logarithmic <- grepl("(^|:)LOG(:|$)", parameters$type)
+    out[logarithmic] <- lapply(out[logarithmic], exp)
+
     return(out)
 }
