@@ -194,7 +194,7 @@ g3_step <- function(step_f, recursing = FALSE, orig_env = environment(step_f)) {
         comment_str <- paste(vapply(tail(x, -1), function (a) {
             if (is.symbol(a)) {
                 # Dereference symbols
-                a <- get(as.character(a), envir = environment(step_f))
+                a <- get(as.character(a), envir = orig_env)
                 # Stocks have a name attribute
                 if (is.list(a) && 'name' %in% names(a)) a <- a$name
             }
@@ -221,7 +221,7 @@ g3_step <- function(step_f, recursing = FALSE, orig_env = environment(step_f)) {
             comment_str <- paste(vapply(tail(x, -2), function (a) {
                 if (is.symbol(a)) {
                     # Dereference symbols
-                    a <- get(as.character(a), envir = environment(step_f))
+                    a <- get(as.character(a), envir = orig_env)
                     # Stocks have a name attribute
                     if (is.list(a) && 'name' %in% names(a)) a <- a$name
                 }
@@ -238,58 +238,114 @@ g3_step <- function(step_f, recursing = FALSE, orig_env = environment(step_f)) {
             inner_f <- call_to_formula(x[[3]], environment(step_f))
             inner_f <- g3_step(inner_f, recursing = TRUE, orig_env = orig_env)
 
-            if (!("length" %in% names(stock$dim))) {
-                # No length dimension, so sum everything
-                out_f <- f_substitute(quote( sum(inner_f) ), list(inner_f = inner_f))
-                environment_merge(environment(step_f), rlang::f_env(out_f))
-                return(rlang::f_rhs(out_f))
-            }
-
             # Find first (stock)__(instance)-named var in inner_f, assume that's source stock.
             source_stock_var <- grep("__", all.vars(inner_f), value = TRUE)
             source_stock_var <- sub('__.*$', '', source_stock_var[[1]])
             source_stock <- get(source_stock_var, envir = orig_env)
-            if (!("length" %in% names(source_stock$dim))) stop("Source stock ", source_stock$name, " has no length, can't resize")
-            source_lg <- g3_stock_def(source_stock, 'minlen')
 
-            dest_lg <- g3_stock_def(stock, 'minlen')
+            if (!("length" %in% names(stock$dim))) {
+                # No length dimension in destination, so sum everything
+                out_f <- f_substitute(quote( sum(inner_f) ), list(inner_f = inner_f))
+            } else if ("length" %in% names(stock$dim) && "length" %in% names(source_stock$dim)) {  # Lengthgroup in source & dest
+                source_lg <- g3_stock_def(source_stock, 'minlen')
+                dest_lg <- g3_stock_def(stock, 'minlen')
 
-            # Get upper bound for length groups, if infinite guess something that should work
-            # NB: Assuming plus-group is one-long is cheating, but probably no easy general answers
-            source_upperlen <- g3_stock_def(source_stock, 'upperlen')
-            if (is.infinite(source_upperlen)) source_upperlen <- tail(source_lg, 1) + 1
-            dest_upperlen <- g3_stock_def(stock, 'upperlen')
-            if (is.infinite(dest_upperlen)) dest_upperlen <- max(tail(source_lg, 1), tail(dest_lg, 1)) + 1
+                # Get upper bound for length groups, if infinite guess something that should work
+                # NB: Assuming plus-group is one-long is cheating, but probably no easy general answers
+                source_upperlen <- g3_stock_def(source_stock, 'upperlen')
+                if (is.infinite(source_upperlen)) source_upperlen <- tail(source_lg, 1) + 1
+                dest_upperlen <- g3_stock_def(stock, 'upperlen')
+                if (is.infinite(dest_upperlen)) dest_upperlen <- max(tail(source_lg, 1), tail(dest_lg, 1)) + 1
 
-            # NB: The force_vector class stops all.equal() from ignoring int/num
-            if (isTRUE(all.equal(hide_force_vector(source_lg), hide_force_vector(dest_lg)))) {
-                # Source == dest, so no point doing a transform
-                out_f <- inner_f
+                # NB: The force_vector class stops all.equal() from ignoring int/num
+                if (isTRUE(all.equal(hide_force_vector(source_lg), hide_force_vector(dest_lg)))) {
+                    # Source == dest, so no point doing a transform
+                    out_f <- inner_f
+                } else {
+                    matrix_name <- paste0(source_stock$name, '_', stock$name, '_lgmatrix')
+
+                    # Formulae to apply matrix
+                    out_f <- f_substitute(quote( as.vector(lg_matrix %*% inner_f) ), list(
+                        lg_matrix = as.symbol(matrix_name),
+                        inner_f = inner_f))
+
+                    # Generate a matrix to transform one to the other
+                    assign(matrix_name, do.call('rbind', lapply(seq_along(dest_lg), function (dest_idx) vapply(seq_along(source_lg), function (source_idx) {
+                        lower_s <- source_lg[[source_idx]]
+                        upper_s <- if (source_idx >= length(source_lg)) source_upperlen else source_lg[[source_idx + 1]]
+
+                        lower_d <- dest_lg[[dest_idx]]
+                        upper_d <- if (dest_idx >= length(dest_lg)) dest_upperlen else dest_lg[[dest_idx + 1]]
+
+                        intersect_size <- min(upper_s, upper_d) - max(lower_s, lower_d)
+                        return(if (intersect_size > 0) intersect_size / (upper_s - lower_s) else 0)
+                    }, numeric(1)))), envir = environment(out_f))
+                }
             } else {
-                matrix_name <- paste0(source_stock$name, '_', stock$name, '_lgmatrix')
-
-                # Formulae to apply matrix
-                out_f <- f_substitute(quote( as.vector(lg_matrix %*% inner_f) ), list(
-                    lg_matrix = as.symbol(matrix_name),
-                    inner_f = inner_f))
-
-                # Generate a matrix to transform one to the other
-                assign(matrix_name, do.call('rbind', lapply(seq_along(dest_lg), function (dest_idx) vapply(seq_along(source_lg), function (source_idx) {
-                    lower_s <- source_lg[[source_idx]]
-                    upper_s <- if (source_idx >= length(source_lg)) source_upperlen else source_lg[[source_idx + 1]]
-
-                    lower_d <- dest_lg[[dest_idx]]
-                    upper_d <- if (dest_idx >= length(dest_lg)) dest_upperlen else dest_lg[[dest_idx + 1]]
-
-                    intersect_size <- min(upper_s, upper_d) - max(lower_s, lower_d)
-                    return(if (intersect_size > 0) intersect_size / (upper_s - lower_s) else 0)
-                }, numeric(1)))), envir = environment(out_f))
+                stop("Cannot convert ", source_stock$name, " with dims ", paste(names(source_stock$dim), collapse = ", "), " to dims ", paste(names(stock$dim), collapse = ", "))
             }
 
             # Add environment to formulae's environment, return inner call
             environment_merge(environment(step_f), rlang::f_env(out_f))
             return(rlang::f_rhs(out_f))
         },
+        # Combine a subpop into the main stock population
+        stock_combine_subpop = function (x) {  # Arguments: stock_ss(stock__num), stock_ss(stock__renewalnum)
+            find_abund_var_name <- function (in_c) {
+                vars <- grep("(\\w+)__(\\w*)num$", all.vars(in_c), value = TRUE)
+                if (length(vars) == 0) stop("No abundance variable in ", deparse1(in_c))
+                return(vars[[1]])
+            }
+            # Extract stock_ss() call (ditching abundance scaling in the process), replace num$ with new_prefix
+            abund_to_measurement <- function(in_c, abund_varname, new_prefix) {
+                # Extract stock_ss() call from in_c
+                ss_c <- f_find(in_c, "stock_ss")
+                if (length(ss_c) == 0) stop("No stock_ss in ", deparse1(in_c))
+                ss_c <- ss_c[[1]]
+
+                # Replace num$ with new_prefix in first argument
+                ss_c[[2]] <- as.symbol(gsub("num$", new_prefix, ss_c[[2]]))
+                return(ss_c)
+            }
+
+            stopifnot(length(x) == 3)
+            mainpop_num_c <- x[[2]]
+            subpop_num_c <- x[[3]]
+
+            mainpop_varname <- find_abund_var_name(mainpop_num_c)
+            mainpop_stockname <- gsub("__.*$", "", mainpop_varname)
+            mainpop_wgt_c <- abund_to_measurement(mainpop_num_c, mainpop_varname, "wgt")
+
+            subpop_varname <- find_abund_var_name(subpop_num_c)
+            subpop_stockname <- gsub("__.*$", "", subpop_varname)
+            subpop_wgt_c <- abund_to_measurement(subpop_num_c, subpop_varname, "wgt")
+
+            # If stocks don't match, wrap in reshape calls
+            if (mainpop_stockname != subpop_stockname) {
+                subpop_num_c <- substitute(stock_reshape(main, x), list(main = mainpop_stockname, x = subpop_num_c))
+                subpop_wgt_c <- substitute(stock_reshape(main, x), list(main = mainpop_stockname, x = subpop_wgt_c))
+            }
+
+            out_c <- substitute({
+                debug_trace("Add result to ", mainpop_stock)
+                mainpop_wgt_c <- ratio_add_pop(
+                    mainpop_wgt_c, mainpop_num_c,
+                    subpop_wgt_c, subpop_num_c)
+                mainpop_num_c <- mainpop_num_c + subpop_num_c
+            }, list(
+                mainpop_stock = as.symbol(mainpop_stockname),
+                mainpop_num_c = mainpop_num_c,
+                mainpop_wgt_c = mainpop_wgt_c,
+                subpop_num_c = subpop_num_c,
+                subpop_wgt_c = subpop_wgt_c,
+                end = NULL ))
+            # Run g3_step again to fix up dependents that got added
+            out_f <- g3_step(out_c, recursing = TRUE, orig_env = orig_env)
+
+            # Add environment to formulae's environment, return inner call
+            environment_merge(environment(step_f), rlang::f_env(out_f))
+            return(rlang::f_rhs(out_f))
+        },
         # stock_ss subsets stock data var, overriding any set expressions
         stock_ss = function (x) { # Arguments: stock data variable (i.e. stock__num), [dim_name = override expr, ...]
             stock_inst <- x[[2]]  # NB: A "stock__num", not "stock"
@@ -426,6 +482,13 @@ g3_step <- function(step_f, recursing = FALSE, orig_env = environment(step_f)) {
 
             return(inner)
         },
+        stock_hasdim = function (x) {  # Arguments: stock variable, dimension name
+            stock_var <- x[[2]]
+            stock <- if (is.symbol(stock_var)) get(as.character(stock_var), envir = orig_env) else NULL
+            dim_name <- x[[3]]
+
+            return(dim_name %in% names(stock$dim))
+        },
         stock_with = function (x) {  # Arguments: stock variable, inner code block
             return(repl_stock_fn(x, 'with'))
         },
@@ -532,8 +595,12 @@ list_to_stock_switch <- function(l, stock_var = "stock") {
 
 # Resolve stock_list (e.g. suitability) ahead of time, unlike list_to_stock_switch
 resolve_stock_list <- function (l, stock) {
+    # Convert numeric vectors into lists
+    if (is.numeric(l))  l <- as.list(l)
+
     # Only one option, return it
     if (!is.list(l)) return(l)
+    if (length(l) == 1) return(l[[1]])
 
     # If the one we want is present, return that
     if (stock$name %in% names(l)) return(l[[stock$name]])
