@@ -36,22 +36,38 @@ actions <- c(actions, ~{
 })
 expecteds$normalize_vec_all_zero_out <- rep(0, 10)
 
-# ratio_add_vec()
-ratio_add_vec_inp_orig_vec <- runif(10) * 100
-ratio_add_vec_inp_orig_amount <- floor(runif(10) * 10)
-ratio_add_vec_inp_new_vec <- runif(10) * 100
-ratio_add_vec_inp_new_amount <- floor(runif(10) * 10)
-ratio_add_vec_output <- rep(0, 10)
+# ratio_add_pop()
+ratio_add_pop_inp_orig_ar <- as.array(runif(10) * 100)
+ratio_add_pop_inp_orig_amount <- as.array(floor(runif(10) * 10))
+ratio_add_pop_inp_new_ar <- as.array(runif(10) * 100)
+ratio_add_pop_inp_new_amount <- as.array(floor(runif(10) * 10))
+ratio_add_pop_output <- rep(0, 10)
+ratio_add_pop_output_derived <- rep(0, 10)
+ratio_add_pop_output_scalar <- 0
 actions <- c(actions, ~{
-    comment('ratio_add_vec')
-    ratio_add_vec_output <- ratio_add_vec(
-        ratio_add_vec_inp_orig_vec, ratio_add_vec_inp_orig_amount,
-        ratio_add_vec_inp_new_vec, ratio_add_vec_inp_new_amount)
-    REPORT(ratio_add_vec_output)
+    comment('ratio_add_pop')
+    ratio_add_pop_output <- ratio_add_pop(
+        ratio_add_pop_inp_orig_ar, ratio_add_pop_inp_orig_amount,
+        ratio_add_pop_inp_new_ar, ratio_add_pop_inp_new_amount)
+    ratio_add_pop_output_derived <- ratio_add_pop(
+        ratio_add_pop_inp_orig_ar, ratio_add_pop_inp_orig_amount,
+        ratio_add_pop_inp_new_ar, ratio_add_pop_inp_new_amount * ratio_add_pop_inp_orig_amount)
+    ratio_add_pop_output_scalar <- ratio_add_pop(
+        ratio_add_pop_inp_orig_ar[[2]], ratio_add_pop_inp_orig_amount[[2]],
+        ratio_add_pop_inp_new_ar[[2]], ratio_add_pop_inp_new_amount[[2]])
+    REPORT(ratio_add_pop_output)
+    REPORT(ratio_add_pop_output_derived)
+    REPORT(ratio_add_pop_output_scalar)
 })
-ratio_add_vec_total <- ratio_add_vec_inp_orig_amount + ratio_add_vec_inp_new_amount
-expecteds$ratio_add_vec_output <- ratio_add_vec_inp_orig_vec * (ratio_add_vec_inp_orig_amount / native_avz(ratio_add_vec_total)) +
-    ratio_add_vec_inp_new_vec * (ratio_add_vec_inp_new_amount / native_avz(ratio_add_vec_total))
+ratio_add_pop_total <- native_avz(ratio_add_pop_inp_orig_amount + ratio_add_pop_inp_new_amount)
+derived_total <- native_avz(ratio_add_pop_inp_orig_amount + (ratio_add_pop_inp_new_amount * ratio_add_pop_inp_orig_amount))
+expecteds$ratio_add_pop_output <- ratio_add_pop_inp_orig_ar * (ratio_add_pop_inp_orig_amount / ratio_add_pop_total) +
+    ratio_add_pop_inp_new_ar * (ratio_add_pop_inp_new_amount / ratio_add_pop_total)
+expecteds$ratio_add_pop_output_derived <-
+    ratio_add_pop_inp_orig_ar * (ratio_add_pop_inp_orig_amount / derived_total) +
+    ratio_add_pop_inp_new_ar * ((ratio_add_pop_inp_new_amount * ratio_add_pop_inp_orig_amount) / derived_total)
+expecteds$ratio_add_pop_output_scalar <- ratio_add_pop_inp_orig_ar[[2]] * (ratio_add_pop_inp_orig_amount[[2]] / ratio_add_pop_total[[2]]) +
+    ratio_add_pop_inp_new_ar[[2]] * (ratio_add_pop_inp_new_amount[[2]] / ratio_add_pop_total[[2]])
 
 # nonconform_mult
 nonconform_inp1 <- array(runif(4*3*2), dim = c(4,3,2))
@@ -128,23 +144,15 @@ expecteds$nonconform_outdiv_avz2a <- nonconform_inp2[,,1] / native_avz(as.vector
 ###############################################################################
 
 nll <- 0.0
-actions <- c(actions, ~{
+actions <- c(actions, gadget3:::g3l_test_dummy_likelihood(), ~{
     comment('done')
-    nll <- nll + g3_param('rv')
     return(nll)
 })
-params <- list(rv=0)
 
 # Compile model
-model_fn <- g3_to_r(actions, trace = FALSE)
-# model_fn <- edit(model_fn)
-if (nzchar(Sys.getenv('G3_TEST_TMB'))) {
-    model_cpp <- g3_to_tmb(actions, trace = FALSE)
-    # model_cpp <- edit(model_cpp)
-    model_tmb <- g3_tmb_adfun(model_cpp, params, compile_flags = c("-O0", "-g"))
-} else {
-    writeLines("# skip: not compiling TMB model")
-}
+model_fn <- g3_to_r(actions)
+model_cpp <- g3_to_tmb(actions)
+params <- attr(model_cpp, "parameter_template")
 
 # Compare everything we've been told to compare
 result <- model_fn(params)
@@ -156,8 +164,4 @@ for (n in ls(expecteds)) {
         tolerance = if (is.null(tolerances[[n]])) 1e-6 else tolerances[[n]] ), n)
 }
 
-if (nzchar(Sys.getenv('G3_TEST_TMB'))) {
-    param_template <- attr(model_cpp, "parameter_template")
-    param_template$value <- params[param_template$switch]
-    gadget3:::ut_tmb_r_compare(model_fn, model_tmb, param_template)
-}
+gadget3:::ut_tmb_r_compare2(model_fn, model_cpp, params)
