@@ -89,14 +89,7 @@ ok_group('g3a_spawn', {
 
     # Compile model
     model_fn <- g3_to_r(all_actions, trace = FALSE)
-    # model_fn <- edit(model_fn)
-    if (nzchar(Sys.getenv('G3_TEST_TMB'))) {
-        model_cpp <- g3_to_tmb(all_actions, trace = FALSE)
-        # model_cpp <- edit(model_cpp)
-        model_tmb <- g3_tmb_adfun(model_cpp, compile_flags = c("-O0", "-g"))
-    } else {
-        writeLines("# skip: not compiling TMB model")
-    }
+    model_cpp <- g3_to_tmb(all_actions, trace = FALSE)
 
     params <- attr(model_fn, 'parameter_template')
 
@@ -189,10 +182,5 @@ ok_group('g3a_spawn', {
         params$hockeystick_r0 * pmin(hist_biomass/params$hockeystick_blim, 1),
         end = NULL ), "hist_mat_g3a_spawn_recruitment_hockeystick__offspringnum: Matches formula")
 
-    # Make sure the inttest model produces identical output in TMB and R
-    if (nzchar(Sys.getenv('G3_TEST_TMB'))) {
-        param_template <- attr(model_cpp, "parameter_template")
-        param_template$value <- params[param_template$switch]
-        gadget3:::ut_tmb_r_compare(model_fn, model_tmb, param_template)
-    }
+    gadget3:::ut_tmb_r_compare2(model_fn, model_cpp, params)
 })
