@@ -15,7 +15,7 @@ obsdata$number <- runif(nrow(obsdata)) * 10000
 
 actions <- list(
     g3a_time(2000, 2005, c(6L, 6L)),
-    g3a_initialconditions(
+    gadget3:::g3a_initialconditions_manual(
         prey_a,
         num_f = g3_formula(stock_ss(prey_a__init), prey_a__init = prey_a__init),
         wgt_f = 10),
@@ -29,7 +29,9 @@ actions <- list(
         obsdata,
         function_f = g3l_distribution_surveyindices_linear(alpha = NULL, beta = 1.8),
         stocks = list(prey_a),
-        report = TRUE))
+        report = TRUE),
+    # NB: Only required for testing
+    gadget3:::g3l_test_dummy_likelihood() )
 actions <- c(actions, list(g3a_report_detail(actions)))
 model_fn <- g3_to_r(actions)
 model_cpp <- g3_to_tmb(actions)
@@ -56,9 +58,4 @@ for (age in unique(obsdata$age)) {
 }
 ok(ut_cmp_equal(as.vector(r), exp_nll), "Total nll matches expected")
 
-if (Sys.getenv('G3_TEST_TMB') == "2") {
-    #model_cpp <- edit(model_cpp)
-    #writeLines(TMB::gdbsource(g3_tmb_adfun(model_cpp, compile_flags = "-g", output_script = TRUE)))
-    model_tmb <- g3_tmb_adfun(model_cpp, trace = TRUE, parameters = params, compile_flags = c("-O0", "-g"))
-    gadget3:::ut_tmb_r_compare(model_fn, model_tmb, params, model_cpp = model_cpp)
-}
+gadget3:::ut_tmb_r_compare2(model_fn, model_cpp, params, g3_test_tmb = 2)
