@@ -105,27 +105,23 @@ ok_group("g3l_distribution_sumofsquares", {
         length = seq(1, 5, by = 1),
         age = 3:4)  # NB: Only report age 3,4
     obsdata$number <- runif(nrow(obsdata))
-    model_fn <- g3_to_r(list(
-        # Keep TMB happy
+    actions <- list(
         g3_formula({
-            nll <- nll + g3_param("dummy", value = 0) 
             REPORT(prey_a__num)
         }),
         g3a_time(2000, 2001),
-        g3a_initialconditions(prey_a,
+        gadget3:::g3a_initialconditions_manual(prey_a,
             num_f = g3_formula(stock_ss(prey_a__init), prey_a__init = prey_a__init),
             wgt_f = 10),
         g3l_abundancedistribution("adist",
             obsdata,
             function_f = g3l_distribution_sumofsquares(over = c("area", "length")),
             stocks = list(prey_a),
-            report = TRUE)))
-    model_cpp <- g3_to_tmb(attr(model_fn, 'actions'), trace = FALSE)
-    if (Sys.getenv('G3_TEST_TMB') == "2") {
-        #model_cpp <- edit(model_cpp)
-        #writeLines(TMB::gdbsource(g3_tmb_adfun(model_cpp, compile_flags = "-g", output_script = TRUE)))
-        model_tmb <- g3_tmb_adfun(model_cpp, trace = TRUE, compile_flags = c("-O0", "-g"))
-    }
+            report = TRUE ),
+        # NB: Only required for testing
+        gadget3:::g3l_test_dummy_likelihood() )
+    model_fn <- g3_to_r(actions)
+    model_cpp <- g3_to_tmb(actions, trace = FALSE)
 
     params <- attr(model_fn, 'parameter_template')
     r <- model_fn(params)
@@ -140,7 +136,8 @@ ok_group("g3l_distribution_sumofsquares", {
         ) ** 2
     }
     ok(ut_cmp_equal(as.numeric(r), expected_nll), "g3l_distribution_sumofsquares statified over length")
-    if (Sys.getenv('G3_TEST_TMB') == "2") gadget3:::ut_tmb_r_compare(model_fn, model_tmb, params, model_cpp = model_cpp)
+
+    gadget3:::ut_tmb_r_compare2(model_fn, model_cpp, params, g3_test_tmb = 2)
 })
 
 ok_group("g3l_distribution:transform_fs", {
@@ -152,9 +149,9 @@ ok_group("g3l_distribution:transform_fs", {
         length = seq(1, 5, by = 1),
         age = 1:5)
     obsdata$number <- runif(nrow(obsdata))
-    model_fn <- g3_to_r(list(
+    actions <- list(
         g3a_time(2000, 2001),
-        g3a_initialconditions(prey_a,
+        gadget3:::g3a_initialconditions_manual(prey_a,
             num_f = g3_formula(stock_ss(prey_a__init), prey_a__init = prey_a__init),
             wgt_f = 10),
         g3l_abundancedistribution("wt",
@@ -183,14 +180,10 @@ ok_group("g3l_distribution:transform_fs", {
             function_f = g3l_distribution_sumofsquares(),
             stocks = list(prey_a),
             report = TRUE),
-        # Keep TMB happy
-        g3_formula( nll <- nll + g3_param("dummy", value = 0) )))
-    model_cpp <- g3_to_tmb(attr(model_fn, 'actions'), trace = FALSE)
-    if (Sys.getenv('G3_TEST_TMB') == "2") {
-        #model_cpp <- edit(model_cpp)
-        #writeLines(TMB::gdbsource(g3_tmb_adfun(model_cpp, compile_flags = "-g", output_script = TRUE)))
-        model_tmb <- g3_tmb_adfun(model_cpp, trace = TRUE, compile_flags = c("-O0", "-g"))
-    }
+        # NB: Only required for testing
+        gadget3:::g3l_test_dummy_likelihood() )
+    model_fn <- g3_to_r(actions)
+    model_cpp <- g3_to_tmb(actions, trace = FALSE)
 
     # Given results / params, apply matrix manually and make sure the results match
     do_test <- function (r, params, msg) {
@@ -217,7 +210,7 @@ ok_group("g3l_distribution:transform_fs", {
     params <- attr(model_fn, 'parameter_template')
     r <- model_fn(params)
     do_test(r, params, "Identity matrix")
-    if (Sys.getenv('G3_TEST_TMB') == "2") gadget3:::ut_tmb_r_compare(model_fn, model_tmb, params, model_cpp = model_cpp)
+    gadget3:::ut_tmb_r_compare2(model_fn, model_cpp, params, g3_test_tmb = 2)
 
     # Length vector applied for len
     ok(ut_cmp_equal(
@@ -232,7 +225,7 @@ ok_group("g3l_distribution:transform_fs", {
     params$reader1matrix[2,] <- c(0, 0.75, 0.25, 0, 0)
     r <- model_fn(params)
     do_test(r, params, "age1, age2 smeared")
-    if (Sys.getenv('G3_TEST_TMB') == "2") gadget3:::ut_tmb_r_compare(model_fn, model_tmb, params, model_cpp = model_cpp)
+    gadget3:::ut_tmb_r_compare2(model_fn, model_cpp, params, g3_test_tmb = 2)
 })
 
 # g3l_distribution_sumofsquaredlogratios
@@ -314,16 +307,16 @@ report_step <- function (var_name, steps, initial_val) {
 
 base_actions <- list(
     g3a_time(1999,2000, step_lengths = c(6, 6), project_years = 0),
-    g3a_initialconditions(prey_a, ~10 * prey_a__midlen, ~100 * prey_a__midlen),
-    g3a_initialconditions(prey_b, ~10 * prey_b__midlen, ~100 * prey_b__midlen),
-    g3a_initialconditions(prey_c, ~10 * prey_c__midlen, ~100 * prey_c__midlen),
+    gadget3:::g3a_initialconditions_manual(prey_a, ~10 * prey_a__midlen, ~100 * prey_a__midlen),
+    gadget3:::g3a_initialconditions_manual(prey_b, ~10 * prey_b__midlen, ~100 * prey_b__midlen),
+    gadget3:::g3a_initialconditions_manual(prey_c, ~10 * prey_c__midlen, ~100 * prey_c__midlen),
     g3a_predate_totalfleet(
         fleet_abc,
         list(prey_a, prey_b, prey_c),
         suitabilities = list(
-            prey_a = ~g3_param_vector("fleet_abc_a") + 0 * stock__midlen,
-            prey_b = ~g3_param_vector("fleet_abc_b") + 0 * stock__midlen,
-            prey_c = ~g3_param_vector("fleet_abc_c") + 0 * stock__midlen),
+            prey_a = ~g3_param_vector("fleet_abc_a", value = rep(0, 10)) + 0 * stock__midlen,
+            prey_b = ~g3_param_vector("fleet_abc_b", value = rep(0, 10)) + 0 * stock__midlen,
+            prey_c = ~g3_param_vector("fleet_abc_c", value = rep(0, 10)) + 0 * stock__midlen),
         amount_f = ~g3_param('amount_ab', value = 100) * area),
     named_list(
         # Capture data just before final step erases it
@@ -373,7 +366,9 @@ base_actions <- list(
 
             # NB: In theory we could inspect the return value, but TMB doesn't give an easy public method for that
             REPORT(nll)
-        }))
+        }),
+        # NB: Only required for testing
+        gadget3:::g3l_test_dummy_likelihood() )
 actions <- c(base_actions, list(
     g3l_catchdistribution(
         'utcd',
@@ -419,20 +414,7 @@ actions <- c(actions, list(
 
 # Compile model
 model_fn <- g3_to_r(actions, trace = FALSE)
-# model_fn <- edit(model_fn)
-if (nzchar(Sys.getenv('G3_TEST_TMB'))) {
-    params <- attr(model_fn, 'parameter_template')
-    params$fleet_abc_a <- c(0, 0, 0, 0.1, 0.2, 0.1, 0, 0, 0, 0)
-    params$fleet_abc_b <- c(0, 0, 0, 0, 0, 0.1, 0.2, 0.1, 0, 0)
-    params$fleet_abc_c <- c(0, 0, 0, 0, 0, 0, 0.1, 0.2, 0.1, 0)
-
-    model_cpp <- g3_to_tmb(actions, trace = FALSE)
-    # model_cpp <- edit(model_cpp)
-    model_tmb <- g3_tmb_adfun(model_cpp, params, compile_flags = c("-O0", "-g"))
-} else {
-    writeLines("# skip: not compiling TMB model")
-    model_cpp <- c()
-}
+model_cpp <- g3_to_tmb(actions, trace = FALSE)
 
 ok_group("Likelihood per step", {
     params <- attr(model_fn, 'parameter_template')
@@ -727,11 +709,7 @@ ok_group("Likelihood per step", {
             log(g3_avoid_zero(r$adist_surveyindices_log_surveyindices_obs__num[,])))**2),
         r$step2_nll)), "step3_nll: Sum of squares, including step2_nll")
 
-    if (nzchar(Sys.getenv('G3_TEST_TMB'))) {
-        param_template <- attr(model_cpp, "parameter_template")
-        param_template$value <- params[param_template$switch]
-        gadget3:::ut_tmb_r_compare(model_fn, model_tmb, param_template)
-    }
+    gadget3:::ut_tmb_r_compare2(model_fn, model_cpp, params)
 })
 
 ok_group("Likelihood per year", {
@@ -807,14 +785,7 @@ ok_group("Likelihood per year", {
 
     # Compile model
     model_fn <- g3_to_r(actions, trace = FALSE)
-    # model_fn <- edit(model_fn)
-    if (nzchar(Sys.getenv('G3_TEST_TMB'))) {
-        model_cpp <- g3_to_tmb(actions, trace = FALSE)
-        # model_cpp <- edit(model_cpp)
-        model_tmb <- g3_tmb_adfun(model_cpp, params, compile_flags = c("-O0", "-g"))
-    } else {
-        writeLines("# skip: not compiling TMB model")
-    }
+    model_cpp <- g3_to_tmb(actions, trace = FALSE)
 
     params <- attr(model_fn, 'parameter_template')
     params$fleet_abc_a <- runif(10, min=0.1, max=0.9)
@@ -1001,9 +972,5 @@ ok_group("Likelihood per year", {
             log(g3_avoid_zero(r$adist_surveyindices_log_surveyindices_obs__num[,])))**2),
         r$step1_nll)), "step3_nll: Sum of squares, including step1_nll")
 
-    if (nzchar(Sys.getenv('G3_TEST_TMB'))) {
-        param_template <- attr(model_cpp, "parameter_template")
-        param_template$value <- params[param_template$switch]
-        gadget3:::ut_tmb_r_compare(model_fn, model_tmb, param_template)
-    }
+    gadget3:::ut_tmb_r_compare2(model_fn, model_cpp, params)
 })
